package java.lang;

import java.io.ObjectInputStream;
import java.io.IOException;
import java.io.ObjectOutputStream;
import java.util.Arrays;
import java.io.ObjectStreamField;
import java.io.Serializable;

public final class StringBuffer extends AbstractStringBuilder implements Serializable, CharSequence
{
    private transient char[] toStringCache;
    static final long serialVersionUID = 3388685877147921107L;
    private static final ObjectStreamField[] serialPersistentFields;
    
    public StringBuffer() {
        super(16);
    }
    
    public StringBuffer(int n) {
        super(n);
    }
    
    public StringBuffer(String s) {
        super(s.length() + 16);
        this.append(s);
    }
    
    public StringBuffer(CharSequence charSequence) {
        this(charSequence.length() + 16);
        this.append(charSequence);
    }
    
    @Override
    public synchronized int length() {
        return this.count;
    }
    
    @Override
    public synchronized int capacity() {
        return this.value.length;
    }
    
    @Override
    public synchronized void ensureCapacity(int n) {
        super.ensureCapacity(n);
    }
    
    @Override
    public synchronized void trimToSize() {
        super.trimToSize();
    }
    
    @Override
    public synchronized void setLength(int length) {
        this.toStringCache = null;
        super.setLength(length);
    }
    
    @Override
    public synchronized char charAt(int n) {
        if (n < 0 || n >= this.count) {
            throw new StringIndexOutOfBoundsException(n);
        }
        return this.value[n];
    }
    
    @Override
    public synchronized int codePointAt(int n) {
        return super.codePointAt(n);
    }
    
    @Override
    public synchronized int codePointBefore(int n) {
        return super.codePointBefore(n);
    }
    
    @Override
    public synchronized int codePointCount(int n, int n2) {
        return super.codePointCount(n, n2);
    }
    
    @Override
    public synchronized int offsetByCodePoints(int n, int n2) {
        return super.offsetByCodePoints(n, n2);
    }
    
    @Override
    public synchronized void getChars(int n, int n2, char[] array, int n3) {
        super.getChars(n, n2, array, n3);
    }
    
    @Override
    public synchronized void setCharAt(int n, char c) {
        if (n < 0 || n >= this.count) {
            throw new StringIndexOutOfBoundsException(n);
        }
        this.toStringCache = null;
        this.value[n] = c;
        return;
    }
    
    @Override
    public synchronized StringBuffer append(Object o) {
        this.toStringCache = null;
        super.append(String.valueOf(o));
        return this;
    }
    
    @Override
    public synchronized StringBuffer append(String s) {
        this.toStringCache = null;
        super.append(s);
        return this;
    }
    
    @Override
    public synchronized StringBuffer append(StringBuffer sb) {
        this.toStringCache = null;
        super.append(sb);
        return this;
    }
    
    @Override
    synchronized StringBuffer append(AbstractStringBuilder abstractStringBuilder) {
        this.toStringCache = null;
        super.append(abstractStringBuilder);
        return this;
    }
    
    @Override
    public synchronized StringBuffer append(CharSequence charSequence) {
        this.toStringCache = null;
        super.append(charSequence);
        return this;
    }
    
    @Override
    public synchronized StringBuffer append(CharSequence charSequence, int n, int n2) {
        this.toStringCache = null;
        super.append(charSequence, n, n2);
        return this;
    }
    
    @Override
    public synchronized StringBuffer append(char[] array) {
        this.toStringCache = null;
        super.append(array);
        return this;
    }
    
    @Override
    public synchronized StringBuffer append(char[] array, int n, int n2) {
        this.toStringCache = null;
        super.append(array, n, n2);
        return this;
    }
    
    @Override
    public synchronized StringBuffer append(boolean b) {
        this.toStringCache = null;
        super.append(b);
        return this;
    }
    
    @Override
    public synchronized StringBuffer append(char c) {
        this.toStringCache = null;
        super.append(c);
        return this;
    }
    
    @Override
    public synchronized StringBuffer append(int n) {
        this.toStringCache = null;
        super.append(n);
        return this;
    }
    
    @Override
    public synchronized StringBuffer appendCodePoint(int n) {
        this.toStringCache = null;
        super.appendCodePoint(n);
        return this;
    }
    
    @Override
    public synchronized StringBuffer append(long n) {
        this.toStringCache = null;
        super.append(n);
        return this;
    }
    
    @Override
    public synchronized StringBuffer append(float n) {
        this.toStringCache = null;
        super.append(n);
        return this;
    }
    
    @Override
    public synchronized StringBuffer append(double n) {
        this.toStringCache = null;
        super.append(n);
        return this;
    }
    
    @Override
    public synchronized StringBuffer delete(int n, int n2) {
        this.toStringCache = null;
        super.delete(n, n2);
        return this;
    }
    
    @Override
    public synchronized StringBuffer deleteCharAt(int n) {
        this.toStringCache = null;
        super.deleteCharAt(n);
        return this;
    }
    
    @Override
    public synchronized StringBuffer replace(int n, int n2, String s) {
        this.toStringCache = null;
        super.replace(n, n2, s);
        return this;
    }
    
    @Override
    public synchronized String substring(int n) {
        return this.substring(n, this.count);
    }
    
    @Override
    public synchronized CharSequence subSequence(int n, int n2) {
        return super.substring(n, n2);
    }
    
    @Override
    public synchronized String substring(int n, int n2) {
        return super.substring(n, n2);
    }
    
    @Override
    public synchronized StringBuffer insert(int n, char[] array, int n2, int n3) {
        this.toStringCache = null;
        super.insert(n, array, n2, n3);
        return this;
    }
    
    @Override
    public synchronized StringBuffer insert(int n, Object o) {
        this.toStringCache = null;
        super.insert(n, String.valueOf(o));
        return this;
    }
    
    @Override
    public synchronized StringBuffer insert(int n, String s) {
        this.toStringCache = null;
        super.insert(n, s);
        return this;
    }
    
    @Override
    public synchronized StringBuffer insert(int n, char[] array) {
        this.toStringCache = null;
        super.insert(n, array);
        return this;
    }
    
    @Override
    public StringBuffer insert(int n, CharSequence charSequence) {
        super.insert(n, charSequence);
        return this;
    }
    
    @Override
    public synchronized StringBuffer insert(int n, CharSequence charSequence, int n2, int n3) {
        this.toStringCache = null;
        super.insert(n, charSequence, n2, n3);
        return this;
    }
    
    @Override
    public StringBuffer insert(int n, boolean b) {
        super.insert(n, b);
        return this;
    }
    
    @Override
    public synchronized StringBuffer insert(int n, char c) {
        this.toStringCache = null;
        super.insert(n, c);
        return this;
    }
    
    @Override
    public StringBuffer insert(int n, int n2) {
        super.insert(n, n2);
        return this;
    }
    
    @Override
    public StringBuffer insert(int n, long n2) {
        super.insert(n, n2);
        return this;
    }
    
    @Override
    public StringBuffer insert(int n, float n2) {
        super.insert(n, n2);
        return this;
    }
    
    @Override
    public StringBuffer insert(int n, double n2) {
        super.insert(n, n2);
        return this;
    }
    
    @Override
    public int indexOf(String s) {
        return super.indexOf(s);
    }
    
    @Override
    public synchronized int indexOf(String s, int n) {
        return super.indexOf(s, n);
    }
    
    @Override
    public int lastIndexOf(String s) {
        return this.lastIndexOf(s, this.count);
    }
    
    @Override
    public synchronized int lastIndexOf(String s, int n) {
        return super.lastIndexOf(s, n);
    }
    
    @Override
    public synchronized StringBuffer reverse() {
        this.toStringCache = null;
        super.reverse();
        return this;
    }
    
    @Override
    public synchronized String toString() {
        if (this.toStringCache == null) {
            this.toStringCache = Arrays.copyOfRange(this.value, 0, this.count);
        }
        return new String(this.toStringCache, true);
    }
    
    private synchronized void writeObject(ObjectOutputStream objectOutputStream) throws IOException {
        ObjectOutputStream.PutField putFields = objectOutputStream.putFields();
        putFields.put("value", this.value);
        putFields.put("count", this.count);
        putFields.put("shared", false);
        objectOutputStream.writeFields();
    }
    
    private void readObject(ObjectInputStream objectInputStream) throws IOException, ClassNotFoundException {
        ObjectInputStream.GetField fields = objectInputStream.readFields();
        this.value = (char[])(char[])fields.get("value", null);
        this.count = fields.get("count", 0);
    }
    
    static {
        serialPersistentFields = new ObjectStreamField[] { new ObjectStreamField("value", char[].class), new ObjectStreamField("count", Integer.TYPE), new ObjectStreamField("shared", Boolean.TYPE) };
    }
}
